#!/bin/bash

# include helper functions script
source /usr/local/bin/helper_functions.sh

if [[ $1 == "hourly" || $1 == "daily" || $1 == "weekly" || $1 == "monthly" ]]; then
	interval="$1"
else
	echo "Unknown interval $1"
	exit 1
fi

config_file="/etc/rsnapshot.conf"
if [[ ! -f "$config_file" ]]; then
	echo "The config file $config_file does not exist"
	exit 1
fi

# mails
address="email@example.org"
success="backup from notebook to server s1 was successful"
error="backup from notebook to server s1 rotaran failed"

# sync_options
rsync_options="--archive --update --delete --compress --delete-excluded --hard-links --partial -e \"ssh\""
backup_source="/backups"
backup_user="backup_notebook"
log_file="/var/log/rsnapshot_backup_to_s1.log"
upload_limit=0     # in kb/s, 0 = unlimited

#########################
echo "backup from client to server s1 started at $(get_timestamp)" >> "$log_file"

# start rsnapshot local backup
rsnapshot $interval
rsnapshot_rc=$?
if [[ $rsnapshot_rc != 0 ]]; then
    msg="rsnapshot $interval backup: error, return code $rsnapshot_rc"
    echo "$msg at $(get_timestamp)" >> "$log_file"
    echo "$msg at $(get_timestamp)" | mail -s "$error" "$address"
    exit 3
fi

# internet connection
if [[ $(is_internet_connection) == "false" ]]; then
    echo "No internet connection available at $(get_timestamp)" >> "$log_file"
    exit 1
fi

# try to contact host
loadme="/tmp/loadme"
# if the test file still exists from the last backup, try to remove it
if [ -e "$loadme" ]; then
	rm "$loadme"
	if [[ $# != 0 ]]; then
		echo "The loadme file can't be removed at $(get_timestamp)" >> "$log_file"
		echo "The loadme file can't be removed at $(get_timestamp)" | mail -s "$error" "$address"
		exit 4
	fi
fi

host="backup_192.168.1.100"
lftp -c "set cmd:at-exit; set net:timeout 10; set net:max-retries 3; \
	get sftp://$backup_user:bla@$host/$backup_user/loadme -o /tmp"
if [ ! -e "$loadme" ]; then
	host="backup_s1"
	lftp -c "set cmd:at-exit; set net:timeout 10; set net:max-retries 3; \
        get sftp://$backup_user:bla@$host/$backup_user/loadme -o /tmp"
	if [ ! -e "$loadme" ]; then
		echo "server s1 not accessible, no backup! at $(get_timestamp)" >> "$log_file"
		echo "server s1 not accessible, no backup! at $(get_timestamp)" | mail -s "$error" "$address"
		exit 2
	fi
fi
rm /tmp/loadme

# sync to server s1
backup_destination="$backup_user@$host:/$backup_user/"
if [[ $upload_limit > 0 ]]; then
    rsync_options="$rsync_options --bwlimit=$upload_limit"
fi

echo "rsync started"
i=0
max_restarts=5
last_exit_code=1
while [ $i -le $max_restarts ]
do
    i=$(( $i + 1 ))
    rsync $rsync_options "$backup_source" "$backup_destination"
    last_exit_code=$?
    if [ $last_exit_code -eq 0 ]; then
        break
    fi
    sleep 300
done

msg=""
if [[ $last_exit_code != 0 ]]; then
    msg="rsync failed after $max_restarts restarts with exit code $last_exit_code"
    echo "$msg at $(get_timestamp)" | mail -s "$error" "$address"
    echo "$msg at $(get_timestamp)" >> "$log_file"
    exit 5
else
    if [ $i -le 1 ]; then
        msg="$interval backup completed successfully"
    else
        msg="$interval backup completed successfully after $i tries"
    fi
    # remove "#" if you also want to get a success mail
    # echo "$msg" | mail -s "$success" "$address"
    echo "$msg at $(get_timestamp)" >> "$log_file"
    exit 0
fi
